# ---------------------------------------------------------------------------
# Copyright 2002 Benjamin Trott
# Copyright 2003 Dwight Shih
#
# Functions for handling and displaying trackback pings
# Derived from the Standalone Trackback package released by Benjamin Trott.
#
# This code is released under the Artistic License.
#
# $Id: service.pm,v 1.1 2003/12/01 13:52:22 dwight Exp $
# ---------------------------------------------------------------------------

package trackback::service;

use strict;
use vars qw($VERSION @ISA);

require POSIX;

use Exporter;
use CGI qw( :cgi );
use File::Spec::Functions;

use trackback::core;

@ISA = qw(Exporter trackback::core);
$VERSION = '1.00';

# ---------------------------------------------------------------------------

sub new {
    my $class = shift;
    my %param = @_;
    my $self  = trackback::core->new(@_);
    bless($self,$class);

    #
    if ( $param{HEADER} ) {
        $self->{HEADER} = $param{HEADER};
    } else {
        $self->{HEADER} = <<HEADER_END;
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">

<html>
<head>
<title>Trackbacks</title>
<base target="_blank" />
<meta name="robots" content="noindex,nofollow">
<style type="text/css">
body {
    color:black;
    background-color:#CCCCCC;
}
.even {
    color:black;
    background-color:white;
    padding-top: 5px;
    padding-bottom: 5px;
}
.odd {
    color:black;
    background-color:#CCCCCC;
    padding-top: 5px;
    padding-bottom: 5px;
}
</style>
</head>
<body>
HEADER_END
    }

    #
    if ( $param{BODY_FUNCTION} ) {
        $self->{BODY_FUNCTION} = $param{BODY_FUNCTION};
    } else {
        $self->{BODY_FUNCTION} = \&_trackback_display;
    }

    #
    if ( $param{FOOTER} ) {
        $self->{FOOTER} = $param{FOOTER};
    } else {
        $self->{FOOTER} = <<FOOTER_END;
</body>
</html>
FOOTER_END
    }

    #
    if ( $param{RSS_FILE} ) {
        $self->{RSS_FILE} = $param{RSS_FILE};
    } else {
        $self->{RSS_FILE} = undef;
    }
    
    return $self;
}

#
#
sub handle {
    my $self = shift;

    my $mode = $self->get_mode();
    my $id   = $self->get_id();
    my $link = param("link");

    if ( $mode ) {
        if ( $mode eq "counts" ) {
            $self->trackback_counts();
        } elsif ( $mode eq "ping" ) {
            $self->trackback_ping( $id );
        } elsif ( $mode eq "list" && $id ) {
            $self->trackback_listing( $id );
        } else {
            $self->exit_with_error( "Bad Command Request" );
        }
    } elsif ( $id ) {
        unless ( $link ) {
            $self->trackback_ping( $id );
        } else {
            $self->trackback_listing( $id );
        }
    } else {
        $self->exit_with_error( "Bad Request" );
    }
}

#
#
sub trackback_counts {
    my $self = shift;
    print <<COUNT_END;
Content-Type: text/html

function trackbackCounter( id ) {
    if (tbcount["t_"+id] ) {
        document.write(tbcount["t_"+id]);
    } else {
        document.write(\"0\");
    }
}
COUNT_END
    print "var tbcount = { ";
    my $count_data = $self->count_load();
    my ($key,$value);
    while ( ($key,$value) = each(%$count_data) ) {
        print "t_$key: $value, ";
    }
    print "tb_null: 0 };\n";
}

#
#
sub trackback_ping {
    my ($self, $id) = @_;
    $self->exit_with_error( sprintf( "No trackback id (%s)",$self->{ID_PARAMETER}) ) 
        unless $id;
    
    my $ping = { map { $_ => scalar param($_) } qw(title excerpt url blog_name) };
    $self->exit_with_error( "No URL (url)" )
        unless $ping->{url};

    $ping->{title} ||= $ping->{url};
    $ping->{timestamp} = time;
    $ping->{ipaddr} = $ENV{REMOTE_ADDR};

    $self->ping_store( $id, $ping );

    if ( $self->{RSS_FILE} && open(FH, ">" . $self->{RSS_FILE}) ) {
        print FH $self->rss_summary();
        close FH;
    }

    $self->exit_with_xml();
}

#
#
sub trackback_listing {
    my $self     = shift;
    my $id       = shift;
    my $data     = $self->trackback_load($id);
    my $ping_url = url() . "/$id";
    my $link     = param("link");

    print "Content-type: text/html\n\n";
    print $self->{HEADER};
    print "The Trackback URL for this post is:<br>$ping_url<p>";
    if ( $link ) {
        print qq(Listed below are links to weblogs that reference <a href="$link">this post</a>:<p>);
    }
    print $self->{BODY_FUNCTION}($data);
    print $self->{FOOTER};
}

sub _trackback_display {
    my ($data) = @_;
    
    my $tmpl = <<TEMPLATE;
<div class="%s">
<div class="trackbackTitle"><b><a href="%s">%s</a></b></div>
<div class="trackbackExcerpt"><b>Excerpt:</b> %s</div>
<div class="trackbackDate"><b>Weblog:</b> %s &#0149; %s</div>
</div>
TEMPLATE

    my $i = 0;
    for my $item (@$data) {
        my $ts = POSIX::strftime("%d %b %Y, %I:%M %p", localtime($item->{timestamp}) );
        printf $tmpl,
            ( $i % 2 ? "odd" : "even"),
            $item->{url}, $item->{title},
            $item->{excerpt} || "[no excerpt]",
            $item->{blog_name} || "[no blog name]",
            $ts;
        $i ++;
    }
}

1;
